// Copyright IBM Corp. 2016. All Rights Reserved.
// Node module: apiconnect-cli-catalogs
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

var path = require('path');
var test = require('tap').test;

var testSupport = require('apiconnect-cli-test-support');
var apic = testSupport.apic({ plugins: [ path.resolve(__dirname, '..') ] });
var logger = testSupport.logger;
var auth = testSupport.auth;

test('Smoke test', function(t) {
  testSupport.basicPluginTest(t, path.dirname(__dirname), {
    catalogs: [
      'list',
      'create <displayName>',
      'get <name>',
      'delete <name>',
    ],
  });

  t.end();
});

var authServer = process.env.APIM_SERVER;
if (!authServer) {
  console.log('\nNo authServer defined.  Exiting\n');
  return;
}

test('Command tests', function(t) {
  t.test('test setup', function(t) {
    return auth.login('ibmapim+steve@gmail.com', '7iron-hide', authServer);
  });

  t.test('invoke catalogs', function(t) {
    logger.clear();
    return apic([ 'catalogs', '--server', authServer, '-O' ])
      .then(function() {
        var output = logger.values(true);
        t.ok(output[ 0 ], 'apic-catalog://sjsldev216.dev.ciondemand.com/orgs/climbon/catalogs/sb');
        t.ok(output[ 1 ], 'apic-catalog://sjsldev216.dev.ciondemand.com/orgs/climbon/catalogs/qa');
        t.ok(output[ 2 ], 'apic-catalog://sjsldev216.dev.ciondemand.com/orgs/climbon/catalogs/staging');
        t.ok(output[ 3 ], 'apic-catalog://sjsldev216.dev.ciondemand.com/orgs/hikeon/catalogs/sb');
        t.ok(output[ 4 ], 'apic-catalog://sjsldev216.dev.ciondemand.com/orgs/hikeon/catalogs/qa');
        t.assert(new RegExp(/^apic-catalog:\/\//).test(output));
        t.end();
      });
  });

  t.test('invoke catalogs with specific organization', function(t) {
    logger.clear();
    return apic([ 'catalogs', '--server', authServer, '-o', 'climbon' ])
      .catch(t.fail)
      .then(function() {
        var output = logger.values(true);
        t.ok(output[ 0 ], 'apic-catalog://sjsldev216.dev.ciondemand.com/orgs/climbon/catalogs/sb');
        t.ok(output[ 1 ], 'apic-catalog://sjsldev216.dev.ciondemand.com/orgs/climbon/catalogs/qa');
        t.ok(output[ 2 ], 'apic-catalog://sjsldev216.dev.ciondemand.com/orgs/climbon/catalogs/staging');
        t.assert(new RegExp(/^apic-catalog:\/\//).test(output));
        t.end();
      });
  });

  t.test('invoke catalogs without organization', function(t) {
    logger.clear();
    return apic([ 'catalogs', '--server', authServer ])
      .catch(function() {
        var output = logger.values(true);
        t.equal(output[0], 'The --organization option is required');
        t.end();
      });
  });

  t.test('invoke catalogs without server', function(t) {
    logger.clear();
    return apic([ 'catalogs' ])
      .then(t.fail)
      .catch(function(err) {
        t.equal(err.message, 'The --server option is required.');
        t.end();
      });
  });

  t.test('create a catalog without display name', function(t) {
    logger.clear();
    return apic([ 'catalogs:create', '--server', authServer, '-o', 'climbon' ])
      .then(t.fail)
      .catch(function(err) {
        t.equal(err.message, 'The parameter `displayName` is required.');
      });
  });

  t.test('create a catalog with --name option and no display name', function(t) {
    logger.clear();
    return apic([ 'catalogs:create', '--name', 'movies', '--server', authServer, '-o', 'climbon' ])
      .then(t.fail)
      .catch(function(err) {
        t.equal(err.message, 'The parameter `displayName` is required.');
      });
  });

  t.test('create a catalog with just a display name', function(t) {
    logger.clear();
    return apic([ 'catalogs:create', 'My Movies', '--server', authServer, '-o', 'climbon' ])
        .catch(t.fail)
        .then(function() {
          var output = logger.values(true);
          t.ok(output[0], 'Create climbon:my-movies');
          t.end();
        });
  });

  t.test('create a catalog with a display name and a name', function(t) {
    logger.clear();
    return apic([ 'catalogs:create', 'My Shows', '--name', 'shows', '--server', authServer, '-o', 'climbon' ])
        .catch(t.fail)
        .then(function() {
          var output = logger.values(true);
          t.ok(output[0], 'Create climbon:shows');
          t.end();
        });
  });

  t.test('Get a catalog', function(t) {
    logger.clear();
    return apic([ 'catalogs:get', 'shows', '--server', authServer, '-o', 'climbon' ])
      .catch(t.fail)
      .then(function() {
        var output = logger.values(true);
        t.ok(output[0], 'display name: My Shows');
        t.ok(output[1], 'name: shows');
        t.ok(output[2], 'organization: climbon');
        t.end();
      });
  });

  t.test('Delete a catalog created using display name', function(t) {
    logger.clear();
    return apic([ 'catalogs:delete', 'my-movies', '--server', authServer, '-o', 'climbon' ])
      .catch(t.fail)
      .then(function() {
        var output = logger.values(true);
        t.ok(output[0], 'Deleted climbon:my-movies');
        t.end();
      });
  });

  t.test('Delete a catalog using display name and name', function(t) {
    logger.clear();
    return apic([ 'catalogs:delete', 'shows', '--server', authServer, '-o', 'climbon' ])
        .catch(t.fail)
        .then(function() {
          var output = logger.values(true);
          t.ok(output[0], 'Deleted climbon:shows');
          t.end();
        });
  });


  t.end();
});

test('test teardown', function(t) {
  return auth.logout(authServer);
});
